#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Au3Check_Parameters=-w 1 -w 2 -w 3 -w 4 -w 5 -w 6
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#include-once

#include <GUIConstantsEx.au3>
#include <WinAPI.au3>
#include <GDIPlus.au3>
#include <Array2D.au3>


; #INDEX# =======================================================================================================================
; Title .........: GdipGui
; AutoIt Version : 3.2.12++
; Language ......: English
; Description ...: This module contains various functions to help displaying graphics from GDI+ on GUIs. You must download the
;                  module "Array2D" available in this page : http://www.autoitscript.com/forum/index.php?showtopic=80195
; ===============================================================================================================================

; #VARIABLES# ===================================================================================================================
Global $GDIPGUI_IMAGE_VALUES[1][7]
; ===============================================================================================================================

; #CURRENT# =====================================================================================================================
;_GdipGui_AddImage
;_GdipGui_AddImageFromFile
;_GdipGui_CreateImage
;_GdipGui_DelImage
;_GdipGui_DisplayImage
;_GdipGui_MoveImage
;_GdipGui_RemoveImages
;_GdipGui_ReplaceImage
; ===============================================================================================================================

; #INTERNAL_USE_ONLY# ===========================================================================================================
;__GdipGui_HWNDGetGraphic
;__GdipGui_ImageGetIndex
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_AddImage
; Description ...: Adds an image on a GUI and refreshes it
; Syntax.........: _GdipGui_AddImage($hWnd, $hImage[, $iLeft = 0[, $iTop = 0[, $iWidth = 0[, $iHeight = 0[, $bRefresh = True]]]]])
; Parameters ....: $hWnd     - Handle to the GUI
;                  $hImage   - Handle to the image
;                  $iLeft    - [optional] Left position of the image on the GUI. Default : 0
;                  $iTop     - [optional] Top position of the image on the GUI. Default : 0
;                  $iWidth   - [optional] Width of the image on the GUI. Default : orginal image's width
;                  $iHeight  - [optional] Height of the image on the GUI. Default : orginal image's height
;                  $bRefresh - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Return values .: Success - 1
;                  Failure - 0, sets @error to :
;                  |1 - $iLeft is invalid
;                  |2 - $iTop is invalid
;                  |3 - $hWnd is not a handle
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_AddImage($hWnd, $hImage, $iLeft = 0, $iTop = 0, $iWidth = 0, $iHeight = 0, $bRefresh = True)
	If $iLeft = Default Then $iLeft = 0
	If $iTop = Default Then $iTop = 0
	If $iWidth = Default Then $iWidth = 0
	If $iHeight = Default Then $iHeight = 0
	If $bRefresh = Default Then $bRefresh = True

	If $iLeft < 0 Or (Not IsInt($iLeft)) Then Return SetError(1, 0, 0)
	If $iTop < 0 Or (Not IsInt($iTop)) Then Return SetError(2, 0, 0)
	If Not IsHWnd($hWnd) Then Return SetError(3, 0, 0)

	; Adds the image to the list
	Local $i = UBound($GDIPGUI_IMAGE_VALUES)
	ReDim $GDIPGUI_IMAGE_VALUES[$i + 1][7]
	$GDIPGUI_IMAGE_VALUES[$i][0] = $hWnd
	$GDIPGUI_IMAGE_VALUES[$i][1] = __GdipGui_HWNDGetGraphic($hWnd)
	$GDIPGUI_IMAGE_VALUES[$i][2] = $hImage
	$GDIPGUI_IMAGE_VALUES[$i][3] = $iLeft
	$GDIPGUI_IMAGE_VALUES[$i][4] = $iTop
	$GDIPGUI_IMAGE_VALUES[$i][5] = $iWidth
	$GDIPGUI_IMAGE_VALUES[$i][6] = $iHeight

	; Refreshes the window
	If $i = 1 Then GUIRegisterMsg(0xF, "__GdipGui_DisplayImage")

	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)

	Return SetError(0, 0, 1)
EndFunc   ;==>_GdipGui_AddImage

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_AddImageFromFile
; Description ...: Loads an image from a file, adds it on a GUI and refreshes it
; Syntax.........: _GdipGui_AddImageFromFile($hWnd, $sFileName[, $iLeft = 0[, $iTop = 0[, $iWidth = 0[, $iHeight = 0[, $bRefresh = True]]]]])
; Parameters ....: $hWnd      - Handle to the GUI
;                  $sFileName - Fully qualified image file name
;                  $iLeft     - [optional] Left position of the image on the GUI. Default : 0
;                  $iTop      - [optional] Top position of the image on the GUI. Default : 0
;                  $iWidth    - [optional] Width of the image on the GUI. Default : orginal image's width
;                  $iHeight   - [optional] Height of the image on the GUI. Default : orginal image's height
;                  $bRefresh  - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Return values .: Success - Handle to the loaded image
;                  Failure - 0, sets @error to :
;                  |1 - $iLeft is invalid
;                  |2 - $iTop is invalid
;                  |3 - $hWnd is not a handle
;                  |4 - Error of file loading
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_AddImageFromFile($hWnd, $sFileName, $iLeft = 0, $iTop = 0, $iWidth = 0, $iHeight = 0, $bRefresh = True)
	If $iLeft = Default Then $iLeft = 0
	If $iTop = Default Then $iTop = 0
	If $iWidth = Default Then $iWidth = 0
	If $iHeight = Default Then $iHeight = 0
	If $bRefresh = Default Then $bRefresh = True

	If $iLeft < 0 Or (Not IsInt($iLeft)) Then Return SetError(1, 0, 0)
	If $iTop < 0 Or (Not IsInt($iTop)) Then Return SetError(2, 0, 0)
	If Not IsHWnd($hWnd) Then Return SetError(3, 0, 0)

	; Loads the image
	Local $i, $hImage
	$hImage = _GDIPlus_ImageLoadFromFile($sFileName)
	If @error Then Return SetError(4, 0, 0)

	; Adds the image to the list
	$i = UBound($GDIPGUI_IMAGE_VALUES)
	ReDim $GDIPGUI_IMAGE_VALUES[$i + 1][7]
	$GDIPGUI_IMAGE_VALUES[$i][0] = $hWnd
	$GDIPGUI_IMAGE_VALUES[$i][1] = __GdipGui_HWNDGetGraphic($hWnd)
	$GDIPGUI_IMAGE_VALUES[$i][2] = $hImage
	$GDIPGUI_IMAGE_VALUES[$i][3] = $iLeft
	$GDIPGUI_IMAGE_VALUES[$i][4] = $iTop
	$GDIPGUI_IMAGE_VALUES[$i][5] = $iWidth
	$GDIPGUI_IMAGE_VALUES[$i][6] = $iHeight

	; Refreshes the window
	If $i = 1 Then GUIRegisterMsg(0xF, "__GdipGui_DisplayImage")

	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)

	Return SetError(0, 0, $hImage)
EndFunc   ;==>_GdipGui_AddImageFromFile

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_CreateImage
; Description ...: Creates an image to use on a GUI
; Syntax.........: _GdipGui_CreateImage(ByRef $hImage, ByRef $hGraphicImage, $iWidth, $iHeight[, $iARGBColor = 0xFFFFFFFF])
; Parameters ....: $hImage        - Variable which will contain the handle of the new image
;                  $hGraphicImage - Variable which will contain the handle of the new image's graphics context
;                  $iWidth        - Width of the image to create
;                  $iHeight       - Height of the image to create
;                  $iARGBColor    - [optional] Color of the image in ARGB. Default : white
; Return values .: Success - 1, $hImage and $hGraphicImage are modified
;                  Failure - 0, sets @error to :
;                  |1 - $iWidth is invalid
;                  |2 - $iHeight is invalid
; Author ........: Tolf
; Modified.......:
; Remarks .......: You are not forced to use the new image on a GUI
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_CreateImage(ByRef $hImage, ByRef $hGraphicImage, $iWidth, $iHeight, $iARGBColor = 0xFFFFFFFF)
	If $iARGBColor = Default Then $iARGBColor = 0xFFFFFFFF

	If $iWidth < 1 Or (Not IsInt($iWidth)) Then Return SetError(1, 0, 0)
	If $iHeight < 1 Or (Not IsInt($iHeight)) Then Return SetError(2, 0, 0)

	Local $hBitmap
	$hBitmap = _WinAPI_CreateBitmap($iWidth, $iHeight, 1, 32)
	$hImage = _GDIPlus_BitmapCreateFromHBITMAP($hBitmap)
	_WinAPI_DeleteObject($hBitmap)
	$hGraphicImage = _GDIPlus_ImageGetGraphicsContext($hImage)
	_GDIPlus_GraphicsClear($hGraphicImage, $iARGBColor)
EndFunc   ;==>_GdipGui_CreateImage

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_DelImage
; Description ...: Deletes an image on a GUI and refreshes it
; Syntax.........: _GdipGui_DelImage($hImage[, $iImageDispose = 0[, $bRefresh = True]])
; Parameters ....: $hWnd          - Handle to the GUI
;                  $hImage        - Handle to the image
;                  $iImageDispose - [optional] If set to 1 then disposes the image. Default : 0
;                  $bRefresh      - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Return values .: Success - 1
;                  Failure - 0, sets @error to :
;                  |1 - $hImage was not registered using _GdipGui_AddImage
;                  |2 - There is no image to delete
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_DelImage($hWnd, $hImage, $iImageDispose = 0, $bRefresh = True)
	If $iImageDispose = Default Then $iImageDispose = 0

	If UBound($GDIPGUI_IMAGE_VALUES) = 1 Then Return SetError(2, 0, 0)

	; Searches the image in the list
	Local $i = __GdipGui_ImageGetIndex($hWnd, $hImage)
	If @error Then Return SetError(1, 0, 0)

	Local $hWndGraphic
	$hWndGraphic = $GDIPGUI_IMAGE_VALUES[$i][1]

	; Deletes the image from the list
	If $iImageDispose Then _GDIPlus_ImageDispose($GDIPGUI_IMAGE_VALUES[$i][2])
	_Array2D_Delete($GDIPGUI_IMAGE_VALUES, $i)

	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)

	If UBound($GDIPGUI_IMAGE_VALUES) = 1 Then
		GUIRegisterMsg(0xF, "")
		_GDIPlus_GraphicsDispose($hWndGraphic)
		Return SetError(0, 0, 1)
	EndIf

	; Delete the window's graphics if necessary
	For $iImage = 1 To UBound($GDIPGUI_IMAGE_VALUES) - 1
		If $GDIPGUI_IMAGE_VALUES[$iImage][0] = $hWnd Then Return SetError(0, 0, 1)
	Next
	_GDIPlus_GraphicsDispose($hWndGraphic)

	Return SetError(0, 0, 1)
EndFunc   ;==>_GdipGui_DelImage

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_DisplayImage
; Description ...: Refreshes the images of a GUI
; Syntax.........: _GdipGui_DisplayImage($hWnd)
; Parameters ....: $hWnd - Handle to the GUI to refresh
; Return values .: None
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_DisplayImage($hWnd)
	GUISetState(@SW_UNLOCK, $hWnd)
	__GdipGui_DisplayImage($hWnd)
EndFunc   ;==>_GdipGui_DisplayImage

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_MoveImage
; Description ...: Refreshes the images of a GUI
; Syntax.........: _GdipGui_MoveImage($hWnd, $hImage[, $iLeft = Default[, $iTop = Default[, $iWidth = Default[, $iHeight = Default[, $bRefresh = True]]]]])
; Parameters ....: $hWnd     - Handle to the GUI
;                  $hImage   - Handle to the image
;                  $iLeft    - [optional] Left position of the image on the GUI. Default : don't change this position
;                  $iTop     - [optional] Top position of the image on the GUI. Default : don't change this position
;                  $iWidth   - [optional] Width of the image on the GUI. Default : don't change this position
;                  $iHeight  - [optional] Height of the image on the GUI. Default : don't change this position
;                  $bRefresh - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Parameters ....: $hWnd - Handle to the GUI
; Return values .: Success - 1
;                  Failure - 0, sets @error to :
;                  |1 - $hImage was not registered using _GdipGui_AddImage
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_MoveImage($hWnd, $hImage, $iLeft = Default, $iTop = Default, $iWidth = Default, $iHeight = Default, $bRefresh = True)
	; Searches the image in the list
	Local $i = __GdipGui_ImageGetIndex($hWnd, $hImage)
	If @error Then Return SetError(1, 0, 0)
	
	; Set the new parameters
	If $iLeft <> Default Then $GDIPGUI_IMAGE_VALUES[$i][3] = $iLeft
	If $iTop <> Default Then $GDIPGUI_IMAGE_VALUES[$i][4] = $iTop
	If $iWidth <> Default Then $GDIPGUI_IMAGE_VALUES[$i][5] = $iWidth
	If $iHeight <> Default Then $GDIPGUI_IMAGE_VALUES[$i][6] = $iHeight
	
	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)
EndFunc   ;==>_GdipGui_MoveImage

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_RemoveImages
; Description ...: Remove all registered on a window
; Syntax.........: _GdipGui_RemoveImages($hWnd[, $bRefresh = True])
; Parameters ....: $hWnd      - Handle to the GUI
;                  $bRefresh  - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Return values .: None
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_RemoveImages($hWnd, $bRefresh = True)
	$j = 0
	$hWndGraphic = 0
	
	; Delete the values in the array
	For $i = 0 To UBound($GDIPGUI_IMAGE_VALUES) - 1
		If $GDIPGUI_IMAGE_VALUES[$i][0] <> $hWnd Then
			$GDIPGUI_IMAGE_VALUES[$j][0] = $GDIPGUI_IMAGE_VALUES[$i][0]
			$j += 1
		ElseIf $hWndGraphic = 0 Then
			$hWndGraphic = $GDIPGUI_IMAGE_VALUES[$i][1]
		EndIf
	Next
	ReDim $GDIPGUI_IMAGE_VALUES[$j + 1][7]
	
	If $hWndGraphic <> 0 Then _GDIPlus_GraphicsDispose($hWndGraphic)
	
	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)
EndFunc   ;==>_GdipGui_RemoveImages

; #FUNCTION# ====================================================================================================================
; Name...........: _GdipGui_ReplaceImage
; Description ...: Replace an image by another
; Syntax.........: _GdipGui_ReplaceImage($hWnd, $hImage, $hNewImage[, $bRefresh = True])
; Parameters ....: $hWnd      - Handle to the GUI
;                  $hImage    - Handle to the image to replace
;                  $hNewImage - Handle to the new image
;                  $bRefresh  - [optional] Set to false if you don't wan't to refresh the image. Default : true
; Return values .: Success - 1
;                  Failure - 0, sets @error to :
;                  |1 - $hImage was not registered using _GdipGui_AddImage
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func _GdipGui_ReplaceImage($hWnd, $hImage, $hNewImage, $bRefresh = True)
	; Searches the image in the list
	Local $i = __GdipGui_ImageGetIndex($hWnd, $hImage)
	If @error Then Return SetError(1, 0, 0)
	
	; Replace the image
	$GDIPGUI_IMAGE_VALUES[$i][2] = $hNewImage
	
	; Refreshes the window
	If $bRefresh Then _GdipGui_DisplayImage($hWnd)
EndFunc   ;==>_GdipGui_ReplaceImage

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: __GdipGui_HWNDGetGraphic
; Description ...: Returns the graphics of a window
; Syntax.........: __GdipGui_HWNDGetGraphic($hWnd)
; Parameters ....: $hWnd - Handle to the GUI
; Return values .: Success - The graphics of the window
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func __GdipGui_HWNDGetGraphic($hWnd)
	Local $iImage

	For $iImage = 1 To UBound($GDIPGUI_IMAGE_VALUES) - 2
		; The window is already registered : returns the graphics' handle
		If $GDIPGUI_IMAGE_VALUES[$iImage][0] = $hWnd Then Return $GDIPGUI_IMAGE_VALUES[$iImage][1]
	Next

	Return _GDIPlus_GraphicsCreateFromHWND($hWnd) ; Creates graphics if necessary
EndFunc   ;==>__GdipGui_HWNDGetGraphic

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: __GdipGui_HWNDGetGraphic
; Description ...: Returns the graphics of a window
; Syntax.........: __GdipGui_HWNDGetGraphic($hWnd)
; Parameters ....: $hWnd   - Handle to the GUI
;                  $hImage - Handle to the image
; Return values .: Success - The index of the pair window/image
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func __GdipGui_ImageGetIndex($hWnd, $hImage)
	Local $i
	For $i = 1 To UBound($GDIPGUI_IMAGE_VALUES) - 1
		If $GDIPGUI_IMAGE_VALUES[$i][0] = $hWnd And $GDIPGUI_IMAGE_VALUES[$i][2] = $hImage Then Return SetError(0, 0, $i)
	Next
	Return SetError(1, 0, 0)
EndFunc   ;==>__GdipGui_ImageGetIndex

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: __GdipGui_DisplayImage
; Description ...: Refreshes the images of a GUI
; Syntax.........: __GdipGui_DisplayImage($hWnd)
; Parameters ....: $hWnd - Handle to the GUI to refresh
; Return values .: None
; Author ........: Tolf
; Modified.......:
; Remarks .......:
; Related .......:
; Link ..........;
; Example .......; Yes
; ===============================================================================================================================
Func __GdipGui_DisplayImage($hWnd)
	Local $iImage

	For $iImage = 1 To UBound($GDIPGUI_IMAGE_VALUES) - 1
		If $GDIPGUI_IMAGE_VALUES[$iImage][0] = $hWnd Then ; The image is on the window
			; resize
			If (($GDIPGUI_IMAGE_VALUES[$iImage][5] > 0) And ($GDIPGUI_IMAGE_VALUES[$iImage][6] > 0)) Then
				_GDIPlus_GraphicsDrawImageRect($GDIPGUI_IMAGE_VALUES[$iImage][1], $GDIPGUI_IMAGE_VALUES[$iImage][2], $GDIPGUI_IMAGE_VALUES[$iImage][3], $GDIPGUI_IMAGE_VALUES[$iImage][4], $GDIPGUI_IMAGE_VALUES[$iImage][5], $GDIPGUI_IMAGE_VALUES[$iImage][6])
			Else
				_GDIPlus_GraphicsDrawImage($GDIPGUI_IMAGE_VALUES[$iImage][1], $GDIPGUI_IMAGE_VALUES[$iImage][2], $GDIPGUI_IMAGE_VALUES[$iImage][3], $GDIPGUI_IMAGE_VALUES[$iImage][4])
			EndIf
		EndIf
	Next
	Return $GUI_RUNDEFMSG ; Return RUNDEF, so that the internal PAINT is called, too
EndFunc   ;==>__GdipGui_DisplayImage